Components.utils.import("resource://modules/logger.jsm");

var EXPORTED_SYMBOLS = ["specificRules"];

var config = {
    obm: {
        autoconfServer: "localhost",
        shouldAutoconfRunWithoutAccountCreation: false,
        mail: {
            incoming: {
                hostname: "localhost",
                type: "imap",
                port: 143,
                socketType: 1,
                auth: Components.interfaces.nsMsgAuthMethod.passwordCleartext
            },
            outgoing: {
                hostname: "localhost",
                port: 25,
                socketType: 1,
                auth: Components.interfaces.nsMsgAuthMethod.none
            }
        }
    },
    ldap: {
        primaryUri: "ldap://localhost/dc=local?cn,mailBox?sub?(mail=%email%)",
        periodicUri: "ldap://localhost/dc=local?cn,mailBox?sub?(mail=%email%)"
    }
}

var configData = {};
var logger = new obmautoconfclientLogger("specificRules");
var specificRules = {

/*
	 using the email and password, this function should return the following structure :
	{
		displayname: "Roger Moore", // the visible name of this account
		incoming: {
			type: "imap", // the incoming mail server protocol 
			hostname: "my.server.com", // the the incoming mail server hostname or IP address
			port: 143, // the incoming mail server port to connect to
			username: "roger", // the username to authenticate
			auth: Components.interfaces.nsMsgAuthMethod.passwordCleartext,
									// 0 is intentionally undefined and invalid
									/// No login needed. E.g. IP-address-based.
									const nsMsgAuthMethodValue none = 1;
									/// Do not use AUTH commands (e.g. AUTH=PLAIN),
									/// but the original login commands that the protocol specified
									/// (POP: "USER"/"PASS", IMAP: "login", not valid for SMTP)
									const nsMsgAuthMethodValue old = 2;
									/// password in the clear. AUTH=PLAIN/LOGIN or old-style login.
									const nsMsgAuthMethodValue passwordCleartext = 3;
									/// hashed password. CRAM-MD5, DIGEST-MD5
									const nsMsgAuthMethodValue passwordEncrypted = 4;
									/// Kerberos / GSSAPI (Unix single-signon)
									const nsMsgAuthMethodValue GSSAPI = 5;
									/// NTLM is a Windows single-singon scheme.
									/// Includes MSN / Passport.net, which is the same with a different name.
									const nsMsgAuthMethodValue NTLM = 6;
									/// Encrypted password or Kerberos / GSSAPI or NTLM.
									/// @deprecated - for migration only.
									const nsMsgAuthMethodValue secure = 8;
									/// Let us pick any of the auth types supported by the server.
									/// Discouraged, because vulnerable to MITM attacks, even if server offers secure auth.
									const nsMsgAuthMethodValue anything = 9;
			deleteByAgeFromServer: false // what's that ?
		},
		outgoing: {
			hostname: "my.server.com", // the the outgoing mail server hostname or IP address
			port: 143, // the outgoing mail server port to connect to
			username: "roger", // the username to authenticate
			auth: Components.interfaces.nsMsgAuthMethod.passwordCleartext
		}

		the callback function takes two arguments : the first one is set if it's an error. If there is no error, the second argument is the structure.
		eg: callback(null, {  });
	}
*/
	initialInformationsRetrieval: function(email, ldaptool, callback) {
		var ldapUrl = config.ldap.primaryUri.replace('%email%', email), data = null;

		try {
			data = ldaptool.query(ldapUrl);
		} catch (e) {
		    Components.utils.reportError(e);
		    return callback("LDAP query error");
		}

		if ( data == null ) {
		    return callback("invalidAddress");
		}

		configData = {
			email: email, // this will be used later on this module, but isn't necessary for account creation
			displayname: data.cn[0],
			incoming: {
				type: config.obm.mail.incoming.type,
				port: config.obm.mail.incoming.port,
				hostname: config.obm.mail.incoming.hostname,
				username: data.mailbox[0],
				socketType: config.obm.mail.incoming.socketType,
				auth: config.obm.mail.incoming.auth,
				deleteByAgeFromServer: false
			},
			outgoing: {
				port: config.obm.mail.outgoing.port,
				hostname: config.obm.mail.outgoing.hostname,
				username: data.mailbox[0],
				socketType: config.obm.mail.outgoing.socketType,
				auth: config.obm.mail.outgoing.auth
			}
		};

		callback(null,configData);
	},

	// when the account is created, and BEFORE launching the HTTP request to the autoconf server
	postAccountCreation: function(accountId, accountManager, prefsManager, callback) {
		if ( accountId ) {
			var serverId = prefsManager.getPref("mail.account."+accountId+".server",null);
			if ( serverId && configData.displayname ) {
				prefsManager.setPref("mail.server."+serverId+".name", configData.displayname ,"user");
			}
			var identities = accountManager.getIdentities(accountId);
			if ( identities && identities.length ) {
				var identity = identities[0];
				var smtpServerId = prefsManager.getPref("mail.identity."+identity+".smtpServer",null);
				if ( smtpServerId ) {
					prefsManager.setPref("mail.smtpserver." + smtpServerId + ".description", "AutoConf SMTP Server");
					prefsManager.setPref("mail.smtpserver." + smtpServerId + ".hostname", configData.outgoing.hostname);
				}
			}

			//record accountId
			var accounts = prefsManager.getPref("extensions.autoconf.managed","");
			if ( !accounts ) {
				accounts = [ accountId ];
			} else {
				accounts = accounts.split(",");
				if ( accounts.indexOf(accountId) < 0 ) {
					accounts.push(accountId);
				}
			}
			prefsManager.setPref("extensions.autoconf.managed",accounts.join(","));
		}

		callback();
	},


	getCIA: function(cia, accountManager, prefsManager, callback ){
		logger.log("getCIA()");
// 		var allAccounts = prefsManager.getPref( "mail.accountmanager.accounts","" ).split(",");
// 		for each( accountId in allAccounts ) {
// 			var obmServer = prefsManager.getPref("mail.account."+accountId+".OBMserver", "" );
// 			var obmEmail = prefsManager.getPref("mail.account."+accountId+".OBMemail", "" );
// 			if ( obmServer.length && obmEmail.length ) {
				// found dedicated autoconfigured account
				var accountsString = prefsManager.getPref("extensions.autoconf.managed","");
// 				logger.log("accounts: "+accounts.join(","));
// 				logger.log("accounts.length: "+accounts.length);
				if ( accountsString.length ) {
					var accounts = accountsString.split(",");
					accountId = accounts[0];
					logger.log("working with account id "+accountId);
					var identities = accountManager.getIdentities(accountId);
					logger.log("identities: " + identities);
					if ( identities.length ) {
						var identityId = identities[0];
						logger.log("working with identity id "+ identityId);
						var identity = accountManager.getIdentityProperties(identityId);
						logger.log("identity: "+ JSON.stringify(identity));
						var smtpServerId = accountManager.getSMTPServerId(identityId);
						logger.log("smtpServerId: "+ smtpServerId );
						var outServerId = accountManager.getServerId(accountId);
						logger.log("outServerId: "+ outServerId);
						
						if ( smtpServerId && outServerId ) {
							var outServer = accountManager.getServerProperties( outServerId );
							logger.log("outServerProperties: "+ JSON.stringify(outServer) );
							var smtpServerHostname = accountManager.getSMTPServerProperties(smtpServerId).hostname;
							var hostname = outServer.realhostname ? outServer.realhostname : outServer.hostname;
							var username = outServer.realUsername ? outServer.realUsername : outServer.Username;
							var fullname = identity.fullName;
							var directoryserver = identity.directoryServer;
							var email = identity.useremail;
							cia.smtpServerHostname = smtpServerHostname;
							cia.hostname = hostname;
							cia.username = username;
							cia.fullname = fullname;
							cia.directoryserver = directoryserver;
							cia.email = email;
							logger.log("CIA email : "+cia.email);
							return callback(null, cia);
						}
						
					}
				}
// 			}
// 		}
		logger.log("CIA email : "+cia.email);
		return callback(null, cia);
	},

	//
	// this one is called one time after the initial account creation
	//
	getInitialAutoconfURL: function(cia, accountManager, prefsManager, ldaptool, callback) {
		logger.log("getInitialAutoconfURL: return URL :"+"http://" + config.obm.autoconfServer + ":8080/obm-autoconf/autoconfiguration/" + cia.email)
		return callback(null, "http://" + config.obm.autoconfServer + ":8080/obm-autoconf/autoconfiguration/" + cia.email, cia);
	},

	getPeriodicAutoconfURL: function(cia, accountManager, prefsManager, ldaptool, callback) {
		logger.log("getPeriodicAutoconfURL: obm url: "+"http://" + config.obm.autoconfServer + ":8080/obm-autoconf/autoconfiguration/" + cia.email);
		return callback(null, "http://" + config.obm.autoconfServer + ":8080/obm-autoconf/autoconfiguration/" + cia.email, cia);
	},

  shouldAutoconfRunWithoutAccountCreation: config.obm.shouldAutoconfRunWithoutAccountCreation
};
